/*
	A basic extension of the java.awt.Dialog class
 */

import java.awt.*;

public class DepositZDialog extends Dialog implements Runnable
{
    // This reference is passed in by the constructor
    MicroFabControl _control;
    
    Thread _thread;
    
	public DepositZDialog(Frame parent, boolean modal)
	{
		super(parent, modal);

		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		//{{INIT_CONTROLS
		setLayout(null);
		setSize(380,96);
		label1 = new java.awt.Label("Z displacement (mm): ",Label.RIGHT);
		label1.setBounds(12,12,132,24);
		add(label1);
		zDisplacement = new java.awt.TextField();
		zDisplacement.setBounds(144,12,96,24);
		add(zDisplacement);
		cancel = new java.awt.Button();
		cancel.setLabel("Cancel");
		cancel.setBounds(264,12,60,24);
		cancel.setBackground(new Color(12632256));
		add(cancel);
		label2 = new java.awt.Label("status: ",Label.RIGHT);
		label2.setBounds(12,36,48,24);
		add(label2);
		status = new java.awt.Label("");
		status.setBounds(60,36,312,24);
		add(status);
		setTitle("Deposit Z");
		//}}

		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymFocus aSymFocus = new SymFocus();
		zDisplacement.addFocusListener(aSymFocus);
		SymAction lSymAction = new SymAction();
		zDisplacement.addActionListener(lSymAction);
		cancel.addActionListener(lSymAction);
		//}}
	}
	
	public void addNotify()
	{
  	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets().left, insets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    // Used for addNotify check.
	boolean fComponentsAdjusted = false;


	public DepositZDialog(Frame parent, String title, boolean modal)
	{
		this(parent, modal);
		setTitle(title);
	}

	public DepositZDialog(Frame parent, MicroFabControl control, boolean modal)
	{
		this(parent, modal);
		_control = control;
	}

    /**
     * Shows or hides the component depending on the boolean flag b.
     * @param b  if true, show the component; otherwise, hide the component.
     * @see java.awt.Component#isVisible
     */
    public void setVisible(boolean b)
	{
		if(b)
		{
			Rectangle bounds = getParent().getBounds();
			Rectangle abounds = getBounds();
	
			setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
				 bounds.y + (bounds.height - abounds.height)/2);
		}
        else {
          // Hiding, so stop the thread
	      if (_thread != null) {
		    _thread.stop();
		    _thread = null;
		    // Thread may have turned on the voltage
            _control.setVTriggerState (0);
		  }
        }

		super.setVisible(b);
	}

	//{{DECLARE_CONTROLS
	java.awt.Label label1;
	java.awt.TextField zDisplacement;
	java.awt.Button cancel;
	java.awt.Label label2;
	java.awt.Label status;
	//}}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == DepositZDialog.this)
				Dialog1_WindowClosing(event);
		}
	}
	
	void Dialog1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);
	}

	class SymFocus extends java.awt.event.FocusAdapter
	{
		public void focusLost(java.awt.event.FocusEvent event)
		{
			Object object = event.getSource();
			if (object == zDisplacement)
				zDisplacement_FocusLost(event);
		}

		public void focusGained(java.awt.event.FocusEvent event)
		{
			Object object = event.getSource();
			if (object == zDisplacement)
				zDisplacement_FocusGained(event);
		}
	}

	void zDisplacement_FocusGained(java.awt.event.FocusEvent event)
	{
		// to do: code goes here.
			 
		//{{CONNECTION
		// Select all of the text
		zDisplacement.selectAll();
		//}}
	}

	void zDisplacement_FocusLost(java.awt.event.FocusEvent event)
	{
		// to do: code goes here.
			 
		//{{CONNECTION
		// Select none of the text
		zDisplacement.select(0,0);
		//}}
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == zDisplacement)
				zDisplacement_EnterHit(event);
			else if (object == cancel)
				cancel_ActionPerformed(event);
		}
	}

	void zDisplacement_EnterHit(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
        if (_thread != null)
          // This really shouldn't happen because we did setEnable(false) for the first thread
          return;
    
		// Disable the TextField
		zDisplacement.setEnabled(false);
				 
		// Highlight the cancel button
		cancel.requestFocus();
		
        // Start the thread
        _thread = new Thread (this);
		_thread.start();
			 
		//{{CONNECTION
		//}}
	}

    public void run ()
    {
		double displacement = Double.valueOf (zDisplacement.getText()).doubleValue();

        depositZHelper (displacement, 2.5, _control, status);
        
		cancel.setLabel("Close");
        // Don't try to close the window inside the thread.  Let the user close it.
    }
    
	void cancel_ActionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		//{{CONNECTION
		// Hide the Dialog
		setVisible(false);
		//}}
	}

    /* Deposit displacement mm in the Z direction.  Write update messages and
         errors to the status Label.  Wait for the given currentThreshold before
         moving the probe.
       This turns the voltage on and off.
       On success, return true, otherwise false if Z is negative, or if we
         time out waiting for the current threshold.
       This is static so it can be used by another dialog.
     */
    static public boolean depositZHelper
      (double displacement, double currentThreshold, MicroFabControl control, Label status)
    {
		// to do: code goes here.
		double current;
        final double mmPerStep = control.MM_PER_REV/control.STEPS_PER_REV;
        boolean error = false;
        int currentWaits = 0;
        
        if (displacement <= 0.) {
          status.setText ("ERROR: Z displacement must be a positive number.");
          return (false);
        }
        
	    // Turn off motor standby.
        control.doutClearBits (control.DOUT_STANDBY);
        
        // Apply voltage
        control.setVTriggerState (2);
        
        status.setText ("preparing...");

        // Wait for capacitive current to settle
        control.waitSeconds (1.);
        
        status.setText ("beginning...");

        // Set up a for loop so we limit the mm moved
        int maxSteps = (int)(displacement/mmPerStep);
        for (int i = 0; i < maxSteps; ++i) {
          // wait for current to rise, up to 60 seconds, determined by adc delay
          int waitCount;
          for (waitCount = (int)(60./control.SECONDS_PER_ADC); waitCount > 0; --waitCount) {
		    current = control.io.adc(control.ADC_CURRENT);
            if (Math.abs (current) > currentThreshold)
              // Current is over the threshold, so continue
              break;
            
            ++currentWaits;
          }
          
          if (waitCount <= 0) {
            // waited too long
            status.setText ("Wait for current expired! Deposited (mm): " + i*mmPerStep);
            error = true;
            break;
          }

          // step clockwise
          control.zStep (false);

		  // use interval as millimeters per step / millimeters per sec
          control.waitSeconds ((control.MM_PER_REV / control.STEPS_PER_REV) / 1.);
          
          if (i % 10 == 0) {
            status.setText ("Current waits: " + currentWaits + ". Deposited (mm): " + i*mmPerStep);
          }
        }
        
        // Stop applying voltage
        control.setVTriggerState (0);

        if (error) {
		  // already did status message
		  return (false);
		}

        status.setText ("Total deposited (mm): " + maxSteps * mmPerStep);
        // Successful
        return (true);
    }

}
