/*
	A basic extension of the java.awt.Dialog class
 */

import java.awt.*;

public class LowerProbeDialog extends Dialog implements Runnable
{
    // This reference is passed in by the constructor
    MicroFabControl _control;

    Thread _thread;

	public LowerProbeDialog(Frame parent, boolean modal)
	{
		super(parent, modal);

		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		//{{INIT_CONTROLS
		setLayout(null);
		setVisible(false);
		setSize(304,116);
		beginLowerProbe = new java.awt.Button();
		beginLowerProbe.setLabel("Begin Lower Probe");
		beginLowerProbe.setBounds(12,12,120,24);
		beginLowerProbe.setBackground(new Color(12632256));
		add(beginLowerProbe);
		cancel = new java.awt.Button();
		cancel.setLabel("Cancel");
		cancel.setBounds(156,12,60,24);
		cancel.setBackground(new Color(12632256));
		add(cancel);
		label1 = new java.awt.Label("status: ",Label.RIGHT);
		label1.setBounds(0,72,48,24);
		add(label1);
		status = new java.awt.Label("");
		status.setBounds(48,72,252,24);
		add(status);
		zeroizeWhenDone = new java.awt.Checkbox("Zeroize Z When Done");
		zeroizeWhenDone.setBounds(24,36,180,24);
		add(zeroizeWhenDone);
		zeroizeWhenDone.setState(true);
		setTitle("Lower Probe");
		//}}

		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		beginLowerProbe.addActionListener(lSymAction);
		cancel.addActionListener(lSymAction);
		//}}
	}

	public void addNotify()
	{
  	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets().left, insets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    // Used for addNotify check.
	boolean fComponentsAdjusted = false;


	public LowerProbeDialog(Frame parent, String title, boolean modal)
	{
		this(parent, modal);
		setTitle(title);
	}

	public LowerProbeDialog(Frame parent, MicroFabControl control, boolean modal)
	{
		this(parent, modal);
		_control = control;
	}

    /**
     * Shows or hides the component depending on the boolean flag b.
     * @param b  if true, show the component; otherwise, hide the component.
     * @see java.awt.Component#isVisible
     */
    public void setVisible(boolean b)
	{
		if(b)
		{
			Rectangle bounds = getParent().getBounds();
			Rectangle abounds = getBounds();

			setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
				 bounds.y + (bounds.height - abounds.height)/2);
		}
        else {
          // Hiding, so stop the thread
	      if (_thread != null) {
		    _thread.stop();
		    _thread = null;
		    // Thread may have turned on the voltage
            _control.setVTriggerState (0);
		  }
        }

		super.setVisible(b);
	}

	//{{DECLARE_CONTROLS
	java.awt.Button beginLowerProbe;
	java.awt.Button cancel;
	java.awt.Label label1;
	java.awt.Label status;
	java.awt.Checkbox zeroizeWhenDone;
	//}}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == LowerProbeDialog.this)
				Dialog1_WindowClosing(event);
		}
	}

	void Dialog1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == beginLowerProbe)
				beginLowerProbe_ActionPerformed(event);
			else if (object == cancel)
				cancel_ActionPerformed(event);
		}
	}

	void beginLowerProbe_ActionPerformed(java.awt.event.ActionEvent event)
	{
        if (_thread != null)
          // This really shouldn't happen because we did setEnable(false) for the first thread
          return;
    
		//{{CONNECTION
		// Disable the Button
		beginLowerProbe.setEnabled(false);
		//}}

		// Highlight the cancel button
		cancel.requestFocus();

		// Disable the Checkbox
		zeroizeWhenDone.setEnabled(false);

	    // Start the thread
        _thread = new Thread (this);
		_thread.start();
    		 
		//{{CONNECTION
		//}}
	}

	void cancel_ActionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		//{{CONNECTION
		// Hide the Dialog
		setVisible(false);
		//}}
	}

    public void run() {
        if (lowerProbeHelper (_control, status)) {
          // We successfully found the surface
          if (zeroizeWhenDone.getState())
            _control.zeroizeZ ();
        }
        
		cancel.setLabel("Close");
        // Don't try to close the window inside the thread.  Let the user close it.
    }

    /* Lower the probe.  Write update messages and errors to the status Label.
       This turns the voltage on and off.
       On success, return true, otherwise false if we reach a maximum displacement
         before contacting the surface.
       This is static so it can be used by another dialog.
     */
    static public boolean lowerProbeHelper (MicroFabControl control, Label status) 
    {
		// to do: code goes here.
        double current = 0.;

	    // Turn off motor standby.
        control.doutClearBits (control.DOUT_STANDBY);

        // Apply voltage
        control.setVTriggerState (1);

        status.setText ("preparing...");

        // Wait for capacitive current to settle
         control.waitSeconds (1.);

        status.setText ("beginning...");

        // With MM_PER_REV of 2 and STEPS_PER_REV of 100000, one step is .00002 mm
        final double mmPerStep = control.MM_PER_REV / control.STEPS_PER_REV;

        // Set up a for loop so we limit the maximum attempts to 1 mm
        int maxCount = (int)(1./mmPerStep);
        int i;
        for (i = 0; i < maxCount; ++i) {
		  current = control.io.adc(control.ADC_CURRENT);
          if (Math.abs (current) > 0.01)
            // Current is above the threshold
            break;

		  // step counterclockwise
		  control.zStep (true);

          if (i % 100 == 0)
            status.setText ("Total lowered (mm): " + (-mmPerStep * i));
        }
        
        // Stop applying voltage
        control.setVTriggerState (0);
        
        if (i < maxCount) {
            status.setText ("Finished. Lowered (mm): " + (-mmPerStep * i));
            return (true);
        }
        else {
            status.setText ("Exceeded max lowering of (mm): " + (-mmPerStep * maxCount));
            return (false);
        }
    }
}
   