/*
	A basic extension of the java.awt.Dialog class
 */

import java.awt.*;

public class MoveZDialog extends Dialog implements Runnable
{
    // This reference is passed in by the constructor
    MicroFabControl _control;
    
    Thread _thread;
    
	public MoveZDialog(Frame parent, boolean modal)
	{
		super(parent, modal);

		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		//{{INIT_CONTROLS
		setLayout(null);
		setVisible(false);
		setSize(283,70);
		label1 = new java.awt.Label("Move Z (mm): ",Label.RIGHT);
		label1.setBounds(0,12,84,24);
		add(label1);
		moveZ = new java.awt.TextField();
		moveZ.setBounds(84,12,84,24);
		add(moveZ);
		cancel = new java.awt.Button();
		cancel.setLabel("Cancel");
		cancel.setBounds(180,12,72,24);
		cancel.setBackground(new Color(12632256));
		add(cancel);
		label2 = new java.awt.Label("status: ",Label.RIGHT);
		label2.setBounds(0,36,84,24);
		add(label2);
		status = new java.awt.Label("");
		status.setBounds(84,36,180,24);
		add(status);
		setTitle("Move Z");
		//}}

		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		moveZ.addActionListener(lSymAction);
		SymFocus aSymFocus = new SymFocus();
		moveZ.addFocusListener(aSymFocus);
		cancel.addActionListener(lSymAction);
		//}}
	}
	
	public void addNotify()
	{
  	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets().left, insets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    // Used for addNotify check.
	boolean fComponentsAdjusted = false;


	public MoveZDialog(Frame parent, String title, boolean modal)
	{
		this(parent, modal);
		setTitle(title);
	}

	public MoveZDialog(Frame parent, MicroFabControl control, boolean modal)
	{
		this(parent, modal);
		_control = control;
	}

    /**
     * Shows or hides the component depending on the boolean flag b.
     * @param b  if true, show the component; otherwise, hide the component.
     * @see java.awt.Component#isVisible
     */
    public void setVisible(boolean b)
	{
		if(b)
		{
			Rectangle bounds = getParent().getBounds();
			Rectangle abounds = getBounds();
	
			setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
				 bounds.y + (bounds.height - abounds.height)/2);
		}
        else {
          // Hiding, so stop the thread
	      if (_thread != null) {
		    _thread.stop();
		    _thread = null;
		  }
        }

		super.setVisible(b);
	}

	//{{DECLARE_CONTROLS
	java.awt.Label label1;
	java.awt.TextField moveZ;
	java.awt.Button cancel;
	java.awt.Label label2;
	java.awt.Label status;
	//}}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == MoveZDialog.this)
				Dialog1_WindowClosing(event);
		}
	}
	
	void Dialog1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == moveZ)
				moveZ_EnterHit(event);
			else if (object == cancel)
				cancel_ActionPerformed(event);
		}
	}

	void moveZ_EnterHit(java.awt.event.ActionEvent event)
	{
        if (_thread != null)
          // This really shouldn't happen because we did setEnable(false) for the first thread
          return;
    
		// to do: code goes here.
		// Disable the TextField
		moveZ.setEnabled(false);

		// Highlight the cancel button
		cancel.requestFocus();

        // Start the thread
        _thread = new Thread (this);
		_thread.start();
			 
		//{{CONNECTION
		//}}
	}

    public void run () {
		// to do: code goes here.
		double mmDisplacement = Double.valueOf (moveZ.getText()).doubleValue();
		double signedSteps = (mmDisplacement/_control.MM_PER_REV) * _control.STEPS_PER_REV;
		int steps;
		boolean ccw;
		
		if (signedSteps < 0.) {
		    ccw = true;
		    steps = (int)(-signedSteps);
		}
		else {
		    ccw = false;
		    steps = (int)signedSteps;
		}
		
	    // Turn off motor standby.
        _control.doutClearBits (_control.DOUT_STANDBY);

		for (int i = 0; i < steps; i++) {
		    _control.zStep (ccw);
		    if (i % 5000 == 0)
              status.setText ("Moved Z (mm): " + ((_control.MM_PER_REV/_control.STEPS_PER_REV) * i));
		}

        status.setText ("Finished. Moved Z (mm): " + mmDisplacement);
        
		cancel.setLabel("Close");
        // Don't try to close the window inside the thread.  Let the user close it.
    }

	class SymFocus extends java.awt.event.FocusAdapter
	{
		public void focusLost(java.awt.event.FocusEvent event)
		{
			Object object = event.getSource();
			if (object == moveZ)
				moveZ_FocusLost(event);
		}

		public void focusGained(java.awt.event.FocusEvent event)
		{
			Object object = event.getSource();
			if (object == moveZ)
				moveZ_FocusGained(event);
		}
	}

	void moveZ_FocusGained(java.awt.event.FocusEvent event)
	{
		// to do: code goes here.
			 
		//{{CONNECTION
		// Select all of the text
		moveZ.selectAll();
		//}}
	}

	void moveZ_FocusLost(java.awt.event.FocusEvent event)
	{
		// to do: code goes here.
			 
		//{{CONNECTION
		// Select none of the text
		moveZ.select(0,0);
		//}}
	}

	void cancel_ActionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
			 
		//{{CONNECTION
		// Hide the Dialog
		setVisible(false);
		//}}
	}
}
